@@ -2,16 +2,31 @@
 Generic delete document chain.
 
 : `projection`
-    The projection used when requesting the document from the database (defaults
-    to None which means the detault projection for the frame class will be
-    used).
+    The projection used when requesting the document from the database
+    (defaults to None which means the detault projection for the frame class
+    will be used).
+
+: `remove_assets`
+    If True then assets associated with the document will be removed when it
+    is deleted. Asset values must be projected as `Asset` instances, the
+    chain supports deleting asset from a value or a list of values (other
+    structures are not supported and must be added manually).
 
 """
 
 import flask
 from manhattan.chains import Chain, ChainMgr
+from manhattan.formatters.text import upper_first
 from manhattan.nav import Nav, NavItem
 
+# Optional imports
+try:
+    from manhattan.assets import Asset
+    __assets_supported__ = True
+
+except ImportError as e:
+    __assets_supported__ = False
+
 from . import factories, utils
 
 __all__ = ['delete_chains']
@@ -40,7 +55,12 @@ delete_chains['post'] = Chain([
 
 
 # Define the links
-delete_chains.set_link(factories.config(projection=None))
+delete_chains.set_link(
+    factories.config(
+        projection=None,
+        remove_assets=False
+    )
+)
 delete_chains.set_link(factories.authenticate())
 delete_chains.set_link(factories.get_document())
 delete_chains.set_link(factories.render_template('delete.html'))
@@ -70,17 +90,20 @@ def decorate(state):
         state.decor['breadcrumbs'].add(
             utils.create_breadcrumb(state.manage_config, 'list')
         )
+
     if Nav.exists(state.manage_config.get_endpoint('view')):
         state.decor['breadcrumbs'].add(
             utils.create_breadcrumb(state.manage_config, 'view', document)
         )
+
     else:
         if Nav.exists(state.manage_config.get_endpoint('update')):
             state.decor['breadcrumbs'].add(
                 utils.create_breadcrumb(
                     state.manage_config,
                     'update',
-                    document
+                    document,
+                    label=upper_first(state.manage_config.name) + ' details'
                 )
             )
 
@@ -105,5 +128,30 @@ def delete_document(state):
         # Doesn't support `logged_delete`
         document.delete()
 
+    if state.remove_assets:
+
+        assert __assets_supported__, \
+            'This link requires manhattan-assets to be installed'
+
+        asset_mgr = flask.current_app.asset_mgr
+
+        # Attempt to delete any assets associated with the document
+        assets_to_remove = []
+        for field in document.get_fields():
+
+            value = document.get(field)
+            if not value:
+                continue
+
+            if isinstance(value, Asset):
+                assets_to_remove.append(value)
+
+            if isinstance(value, list) and isinstance(value[0], Asset):
+                for a in value:
+                    if isinstance(a, Asset):
+                        assets_to_remove.append(a)
+
+        asset_mgr.remove_many(assets_to_remove)
+
     # Flash message that the document was deleted
     flask.flash('{document} deleted.'.format(document=document))
